import { TestingModule, Test } from '@nestjs/testing';
import { DatabaseModule } from '../../../database/database.module';
import { PrismaService } from '../../../database/src/adapters/secondaries/prisma-service';
import { DatabaseException } from '../../../database/src/exceptions/DatabaseException';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { v4 } from 'uuid';
import * as bcrypt from 'bcrypt';
import { Auth } from '../../domain/entities/auth';

describe('AuthRepository', () => {
  let prismaService: PrismaService;
  let authRepository: AuthRepository;

  const createAuths = async (nbToCreate = 10) => {
    for (let i = 0; i < nbToCreate; i++) {
      await prismaService.auth.create({
        data: {
          uuid: v4(),
          password: bcrypt.hashSync(`password-${i}`, 10),
        },
      });
    }
  };

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [DatabaseModule],
      providers: [AuthRepository, PrismaService],
    }).compile();

    prismaService = module.get<PrismaService>(PrismaService);
    authRepository = module.get<AuthRepository>(AuthRepository);
  });

  afterAll(async () => {
    await prismaService.$disconnect();
  });

  beforeEach(async () => {
    await prismaService.auth.deleteMany();
  });

  describe('findAll', () => {
    it('should return an empty data array', async () => {
      const res = await authRepository.findAll();
      expect(res).toEqual({
        data: [],
        total: 0,
      });
    });

    it('should return a data array with 8 auths', async () => {
      await createAuths(8);
      const auths = await authRepository.findAll();
      expect(auths.data.length).toBe(8);
      expect(auths.total).toBe(8);
    });

    it('should return a data array limited to 10 auths', async () => {
      await createAuths(20);
      const auths = await authRepository.findAll();
      expect(auths.data.length).toBe(10);
      expect(auths.total).toBe(20);
    });
  });

  describe('findOneByUuid', () => {
    it('should return an auth', async () => {
      const authToFind = await prismaService.auth.create({
        data: {
          uuid: v4(),
          password: bcrypt.hashSync(`password`, 10),
        },
      });

      const auth = await authRepository.findOneByUuid(authToFind.uuid);
      expect(auth.uuid).toBe(authToFind.uuid);
    });

    it('should return null', async () => {
      const auth = await authRepository.findOneByUuid(
        '544572be-11fb-4244-8235-587221fc9104',
      );
      expect(auth).toBeNull();
    });
  });

  describe('create', () => {
    it('should create an auth', async () => {
      const beforeCount = await prismaService.auth.count();

      const authToCreate: Auth = new Auth();
      authToCreate.uuid = v4();
      authToCreate.password = bcrypt.hashSync(`password`, 10);
      const auth = await authRepository.create(authToCreate);

      const afterCount = await prismaService.auth.count();

      expect(afterCount - beforeCount).toBe(1);
      expect(auth.uuid).toBeDefined();
    });
  });

  describe('update', () => {
    it('should update auth password', async () => {
      const authToUpdate = await prismaService.auth.create({
        data: {
          uuid: v4(),
          password: bcrypt.hashSync(`password`, 10),
        },
      });

      const toUpdate: Auth = new Auth();
      toUpdate.password = bcrypt.hashSync(`newPassword`, 10);
      const updatedAuth = await authRepository.update(
        authToUpdate.uuid,
        toUpdate,
      );

      expect(updatedAuth.uuid).toBe(authToUpdate.uuid);
    });

    it('should throw DatabaseException', async () => {
      const toUpdate: Auth = new Auth();
      toUpdate.password = bcrypt.hashSync(`newPassword`, 10);

      await expect(
        authRepository.update('544572be-11fb-4244-8235-587221fc9104', toUpdate),
      ).rejects.toBeInstanceOf(DatabaseException);
    });
  });

  describe('delete', () => {
    it('should delete an auth', async () => {
      const authToRemove = await prismaService.auth.create({
        data: {
          uuid: v4(),
          password: bcrypt.hashSync(`password`, 10),
        },
      });
      await authRepository.delete(authToRemove.uuid);

      const count = await prismaService.auth.count();
      expect(count).toBe(0);
    });

    it('should throw DatabaseException', async () => {
      await expect(
        authRepository.delete('544572be-11fb-4244-8235-587221fc9104'),
      ).rejects.toBeInstanceOf(DatabaseException);
    });
  });
});
